/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)connext_cpp_version.h    generated by: makeheader    Sat Mar  9 09:50:32 2024
 *
 *		built from:	version.ifcxx
 */

#ifndef connext_cpp_version_h
#define connext_cpp_version_h



#include <string>
#include <iostream>

#include "log/log_makeheader.h"
#include "dds_c/dds_c_infrastructure.h"

#ifndef xmq_cpp_dll_h
  #include "connext_cpp/connext_cpp_dll.h"
#endif

namespace connext {

/*e
 * @ingroup PatternUtilitiesModule
 * @brief The Connext Messaging Library version

 * This class represents the Connext Messaging library version.
 * The version format is \<major\>.\<minor\>.\<release\>.\<revision\>.
 */
class XMQCPPDllExport MessagingLibraryVersion {
public:
    MessagingLibraryVersion();

    /*e
     * @brief Get the library major version digit
     */
    DDS_Char major_version() const {
        return _version.major;
    };

    /*e
     * @brief Get the library minor version digit
     */
    DDS_Char minor_version() const {
        return _version.minor;
    };

    /*e
     * @brief Get the library release version digit
    */
    DDS_Char release_version() const {
        return _version.release;
    };

    /*e
     * @brief Get the library revision version digit
    */
    DDS_Char build_version() const {
        return _version.revision;
    };

private:
    DDS_ProductVersion_t _version;
};

XMQCPPDllExport
std::ostream &operator<<(
        std::ostream &os,
        const MessagingLibraryVersion &version);

/*e
 * @ingroup PatternUtilitiesModule
 * @brief The Connext Messaging version
 */
class XMQCPPDllExport MessagingVersion {
public:

    /*e
     * @brief Get the API version
    */
    static const MessagingLibraryVersion &get_api_version();

    /*e
     * @brief Get the API version as a string
    */
    static const std::string &get_api_version_string();

    /*e
     * @brief Get the build version as a string
    */
    static const std::string &get_api_build_version();
};

};



#endif /* connext_cpp_version_h */
