/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)event_generator.h    generated by: makeheader    Sat Mar  9 09:37:52 2024
 *
 *		built from:	generator.ifc
 */

#ifndef event_generator_h
#define event_generator_h

struct RTIClock;

struct RTINtpTime;

struct REDAWorker;

struct RTIEventGeneratorListener;

struct RTIEventGenerator;


  #ifndef osapi_type_h
    #include "osapi/osapi_type.h"
  #endif
  #ifndef osapi_rtpsGuid_h
    #include "osapi/osapi_rtpsGuid.h"
  #endif
  #ifndef advlog_context_h
    #include "advlog/advlog_context.h"
  #endif
  #ifndef osapi_ntptime_h
    #include "osapi/osapi_ntptime.h"
  #endif
  #ifndef reda_fastBuffer_h
    #include "reda/reda_fastBuffer.h"
  #endif

#ifdef __cplusplus
    extern "C" {
#endif


/* RTPS 2.0: match larger MIG interpreter storage */
/* CORE-13167 - Same size on 32/64 bit architectures. */
#ifdef RTI_64BIT
  #define RTI_EVENT_GENERATOR_LISTENER_STORAGE_LENGTH (8)
#else
  #define RTI_EVENT_GENERATOR_LISTENER_STORAGE_LENGTH (16)
#endif

struct RTIEventGeneratorListenerStorage {
    void *field[RTI_EVENT_GENERATOR_LISTENER_STORAGE_LENGTH];
};


#define RTIEventGeneratorListenerStorage_INITIALIZER \
    { \
        {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL} /* field */ \
    }

typedef RTIBool (*RTIEventGeneratorListenerOnEventCallback)(
    const struct RTIEventGeneratorListener *me,
    struct RTINtpTime *newTime, struct RTINtpTime *newSnooze,
    const struct RTINtpTime *now, const struct RTINtpTime *time,
    const struct RTINtpTime *snooze,
    const struct RTIEventGeneratorListenerStorage *listenerStorage,
    struct REDAWorker *worker);

struct RTIEventGeneratorListener {
    /*e Generator will call this method for the fired event */
    RTIEventGeneratorListenerOnEventCallback onEvent;
};

struct RTIEventGeneratorProperty {
    /*e For performance, the generator creates events from a fast buffer pool
       to avoid allocating memory dynamically.  The fast buffer needs a hint
       for the initial pool size.  If there will be many events, the initial
       pool should be appropriately large.
    */
    struct REDAFastBufferPoolGrowthProperty growth;
    /*i
     * Time Based Logging.
     * Print a warning message when the event start/execution is delayed more
     * time than the threashold.
     * Default: INFINITE
     */
    RTINtpTime eventTimeThreshold;
};


#define RTI_EVENT_GENERATOR_PROPERTY_DEFAULT \
{ \
    {100, REDA_FAST_BUFFER_POOL_UNLIMITED, REDA_FAST_BUFFER_POOL_UNLIMITED}, \
    {RTI_NTP_TIME_SEC_MAX, RTI_NTP_TIME_FRAC_AUTO} /* eventTimeThreshold */ \
}


#define RTI_EVENT_GENERATOR_POST_NEW_EVENT (0)


#define RTI_EVENT_GENERATOR_CONTEXT_PARAMS_MAX_SIZE 5


struct RTIEventGeneratorActivityContextStorage {
    /* It is the objectId of the resource doing the activity for each event. */
    RTI_UINT32 activityContextResourceObjectId;
    /*
     * The activity entry is:
     * - Filled in the postEvent method.
     * - Entered and left before and after calling the onEvent method.
     */
    struct RTIOsapiActivityContextStackEntry activityContextEntry;
    /* The list of params of the activityContextEntry */
    void *activityContextEntryParams[RTI_EVENT_GENERATOR_CONTEXT_PARAMS_MAX_SIZE];
};



#define RTIEventGeneratorActivityContextStorage_INITIALIZER \
{ \
    RTI_OSAPI_RTPS_OBJECT_ID_UNKNOWN, /* activityContextResourceObjectId*/ \
    RTIOsapiActivityContextStackEntry_initialize( \
        RTI_OSAPI_ACTIVITY_CONTEXT_ENTRY_KIND_ACTIVITY), /* activityContextEntry */ \
    {NULL, NULL} /* activityContextEntryParams */ \
}

typedef RTIBool (*RTIEventGeneratorEventPostFunction)(
        struct RTIEventGenerator *me,
        const struct RTINtpTime *time,
        const struct RTINtpTime *snooze,
        const struct RTIEventGeneratorListener *listener,
        const struct RTIEventGeneratorListenerStorage *listenerStorage,
        int bytesToCompareIfReuse,
        const struct RTIEventGeneratorActivityContextStorage *activityContextStorage);

struct RTIEventGenerator {
    /*e Schedule future event */
    RTIEventGeneratorEventPostFunction postEvent;
};


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#include "event/event_generator_impl.h"

#endif /* event_generator_h */
