/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)event_generator_impl.h    generated by: makeheader    Sat Mar  9 09:37:52 2024
 *
 *		built from:	generator_impl.ifc
 */

#ifndef event_generator_impl_h
#define event_generator_impl_h


#ifdef __cplusplus
    extern "C" {
#endif


/* ----------------------------------------------------------------------- */

/*i \ingroup RTIEventGeneratorClass
 *
 * Sets an event's activityStorage with contextual information. The Event thread
 * will include this contextual information when logging a message for the
 * associated event.
 *
 * This API adds an activity without parameters. If you need to pass extra
 * parameters, use RTIEventGeneratorActivityContext_setWithParams instead.
 *
 * @param activityStorage__ \b In. Struct to save the activity context.
 * @param localObjectId__ \b In. The objectId of the local resource entry. Pass
 * MIG_RTPS_OBJECT_ID_UNKNOWN if not known.
 * @param format__ \b In. The format of the activity entry.
 */
#define RTIEventGeneratorActivityContext_set( \
        activityStorage__, \
        localObjectId__, \
        format__) \
    (activityStorage__)->activityContextResourceObjectId = (localObjectId__); \
    (activityStorage__)->activityContextEntry.kind = \
            RTI_OSAPI_ACTIVITY_CONTEXT_ENTRY_KIND_ACTIVITY; \
    (activityStorage__)->activityContextEntry.format = (format__)

/*i \ingroup RTIEventGeneratorClass
 *
 * Sets an event's activityStorage with contextual information. The Event thread
 * will include this contextual information when logging a message for the
 * associated event.
 *
 * This API adds an activity with parameters (maximum of 4 parameters).
 * If you don't need to pass extra parameters, use
 * RTIEventGeneratorActivityContext_set instead to reduce the impact on
 * performance.
 *
 * @param activityStorage__ \b In. Struct to save the activity context.
 * @param localObjectId__ \b In. The objectId of the local resource entry. Pass
 * MIG_RTPS_OBJECT_ID_UNKNOWN if not known.
 * @param format__ \b In. The format of the activity entry.
 * @param ... \b In. List of params of the activity entry.
 */
#define RTIEventGeneratorActivityContext_setWithParams( \
        activityStorage__, \
        localObjectId__, \
        format__, \
        ...) \
{ \
    RTI_UINT32 __actAttrListIndex = 0; \
    (activityStorage__)->activityContextResourceObjectId = (localObjectId__); \
    (activityStorage__)->activityContextEntry.kind = \
            RTI_OSAPI_ACTIVITY_CONTEXT_ENTRY_KIND_ACTIVITY_WITH_PARAMS; \
    (activityStorage__)->activityContextEntry.format = (format__); \
    /* \
     * Does not matter if the function fails. This is just to include extra \
     * information on the logging messages. The result of this function should \
     * not change the behaviour of the process. \
     */ \
    RTIOsapiUtility_unusedReturnValue( \
            RTIOsapiActivityContext_getParamList( \
                    (activityStorage__)->activityContextEntryParams, \
                    &__actAttrListIndex, \
                    RTI_EVENT_GENERATOR_CONTEXT_PARAMS_MAX_SIZE, \
                    (format__), \
                    __VA_ARGS__), \
            RTIBool); \
}

/* ----------------------------------------------------------------------- */


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* event_generator_impl_h */
