/* $Id$

(c) Copyright, Real-Time Innovations, 2022.
All rights reserved.

No duplications, whole or partial, manual or electronic, may be made
without express written permission.  Any such copies, or
revisions thereof, must display this notice unaltered.
This code contains trade secrets of Real-Time Innovations, Inc.
============================================================================= */

#ifndef OMG_RTI_DDS_DOMAIN_DISCOVERY_HPP_
#define OMG_RTI_DDS_DOMAIN_DISCOVERY_HPP_

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#include <dds/core/corefwd.hpp>
#include <dds/domain/DomainParticipant.hpp>

namespace rti { namespace domain {

/**
 * @relatesalso dds::domain::DomainParticipant
 * @headerfile rti/domain/discovery.hpp
 *
 * @brief @extension Prevents ignored remote DomainParticipants from
 * receiving traffic from the local DomainParticipant.
 *
 * \p \#include \p <rti/domain/discovery.hpp>
 *
 * @namespace_note rti::domain
 *
 * \dref_details_DomainParticipant_banish_ignored_participants 
 */
OMG_DDS_API
void banish_ignored_participants(
        const dds::domain::DomainParticipant &participant);

/**
 * @relatesalso dds::domain::DomainParticipant 
 * @headerfile rti/domain/discovery.hpp 
 *
 * @brief @extension Returns rti::core::policy::EntityName::name for the
 *        specified DomainParticipant.
 *
 * \p \#include \p <rti/domain/discovery.hpp>
 *
 * @namespace_note rti::domain
 *
 * @details \dref_details_DomainParticipant_get_discovered_participant_subject_name
 */
OMG_DDS_API
rti::core::optional_value<std::string> discovered_participant_subject_name(
        const dds::domain::DomainParticipant &participant,
        const dds::core::InstanceHandle &participant_handle);

/**
 * @relatesalso dds::domain::DomainParticipant
 * @headerfile rti/domain/discovery.hpp
 *
 * @brief @extension Returns a list of discovered DomainParticipant entities
 *        that have the given rti::core::policy::EntityName::name.
 *
 * \p \#include \p <rti/domain/discovery.hpp>
 *
 * @namespace_note rti::domain
 *
 * \dref_details_DomainParticipant_get_discovered_participants_from_subject_name
 */
OMG_DDS_API
dds::core::InstanceHandleSeq discovered_participants_from_subject_name(
        const dds::domain::DomainParticipant &participant,
        const rti::core::optional_value<std::string> &subject_name);

} }

#endif // OMG_RTI_DDS_DOMAIN_DISCOVERY_HPP_
