/* $Id$

(c) Copyright, Real-Time Innovations, 2022.
All rights reserved.

No duplications, whole or partial, manual or electronic, may be made
without express written permission.  Any such copies, or
revisions thereof, must display this notice unaltered.
This code contains trade secrets of Real-Time Innovations, Inc.
*/

#ifndef RTI_DDS_TOPIC_TRUST_PARTICIPANT_TRUST_KEY_ESTABLISHMENT_ALGORITHM_INFO_HPP_
#define RTI_DDS_TOPIC_TRUST_PARTICIPANT_TRUST_KEY_ESTABLISHMENT_ALGORITHM_INFO_HPP_

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#include "dds_c/dds_c_infrastructure_impl.h"

#include <dds/core/types.hpp>
#include <dds/core/Value.hpp>
#include <dds/core/Exception.hpp>
#include <rti/core/NativeValueType.hpp>
#include <rti/topic/trust/TrustAlgorithmRequirements.hpp>

namespace rti { namespace topic { namespace trust {

class ParticipantTrustKeyEstablishmentAlgorithmInfoAdapter {
public:
    typedef struct DDS_ParticipantTrustKeyEstablishmentAlgorithmInfo native_type;

    static void initialize(native_type& native_value)
    {
        static const native_type DEFAULT =
                DDS_ParticipantTrustKeyEstablishmentAlgorithmInfo_INITIALIZER_NOSEC;
        native_value = DEFAULT;
    }

    static void finalize(native_type&)
    {
        // nothing to do
    }

    static void copy(native_type& destination, const native_type& source)
    {
        DDS_ParticipantTrustKeyEstablishmentAlgorithmInfo_copy(&destination, &source);
    }

    static bool equals(const native_type& first, const native_type& second)
    {
        return DDS_ParticipantTrustKeyEstablishmentAlgorithmInfo_equals(&first, &second)
                == DDS_BOOLEAN_TRUE;
    }
};

class ParticipantTrustKeyEstablishmentAlgorithmInfo;

} } // namespace topic::trust

namespace core {

template <>
struct native_type_traits<rti::topic::trust::ParticipantTrustKeyEstablishmentAlgorithmInfo> {
    typedef rti::topic::trust::ParticipantTrustKeyEstablishmentAlgorithmInfoAdapter adapter_type;
    typedef DDS_ParticipantTrustKeyEstablishmentAlgorithmInfo native_type;
};

}

namespace topic { namespace trust {

/**
 * @ingroup DDSBuiltInTopicCommonModule
 * @RTI_class_definition class ParticipantTrustKeyEstablishmentAlgorithmInfo
 * @headerfile rti/topic/trust/ParticipantTrustKeyEstablishmentAlgorithmInfo.hpp
 *
 * @brief @extension @st_value_type Trust Plugins key establishment algorithm
 * information associated with the discovered DomainParticipant.
 *
 * @details \dref_details_ParticipantTrustKeyEstablishmentAlgorithmInfo
 */
class OMG_DDS_API ParticipantTrustKeyEstablishmentAlgorithmInfo : public rti::core::NativeValueType<ParticipantTrustKeyEstablishmentAlgorithmInfo> {
public:
    RTI_NATIVE_VALUE_TYPE_DEFINE_DEFAULT_MOVE_OPERATIONS(ParticipantTrustKeyEstablishmentAlgorithmInfo)

    typedef rti::core::NativeValueType<ParticipantTrustKeyEstablishmentAlgorithmInfo> Base;
    /**
     * @brief Create an instance with the default Trust Key Establishment
     * Algorithm Info associated with the discovered DomainParticipant.
     *
     *  @details \dref_details_ParticipantTrustKeyEstablishmentAlgorithmInfo_new
     */
    ParticipantTrustKeyEstablishmentAlgorithmInfo() = default;

    ParticipantTrustKeyEstablishmentAlgorithmInfo(
            const struct DDS_ParticipantTrustKeyEstablishmentAlgorithmInfo& native_algo)
      : Base(native_algo)
    {
    }

    /**
     * @brief Get the Trust Plugins key establishment algorithm requirements of
     * the DomainParticipant in the context of deriving a shared secret.
     *
     * @details \dref_details_ParticipantTrustKeyEstablishmentAlgorithmInfo_shared_secret
     */
    TrustAlgorithmRequirements shared_secret() const
    {
        return native().shared_secret;
    }
};

} } }

#endif // RTI_DDS_TOPIC_TRUST_PARTICIPANT_TRUST_KEY_ESTABLISHMENT_ALGORITHM_INFO_HPP_
