/* $Id$

(c) Copyright, Real-Time Innovations, 2022.
All rights reserved.

No duplications, whole or partial, manual or electronic, may be made
without express written permission.  Any such copies, or
revisions thereof, must display this notice unaltered.
This code contains trade secrets of Real-Time Innovations, Inc.
*/

#ifndef RTI_DDS_TOPIC_TRUST_ALGORITHM_REQUIREMENTS_HPP_
#define RTI_DDS_TOPIC_TRUST_ALGORITHM_REQUIREMENTS_HPP_

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#include "dds_c/dds_c_infrastructure_impl.h"

#include <dds/core/types.hpp>
#include <dds/core/Value.hpp>
#include <dds/core/Exception.hpp>
#include <rti/core/NativeValueType.hpp>

namespace rti { namespace topic { namespace trust {

class TrustAlgorithmRequirementsAdapter {
public:
    typedef struct DDS_TrustAlgorithmRequirements native_type;

    static void initialize(native_type& native_value)
    {
        static const native_type DEFAULT =
                DDS_TrustAlgorithmRequirements_INITIALIZER;
        native_value = DEFAULT;
    }

    static void finalize(native_type&)
    {
        // nothing to do
    }

    static void copy(native_type& destination, const native_type& source)
    {
        DDS_TrustAlgorithmRequirements_copy(&destination, &source);
    }

    static bool equals(const native_type& first, const native_type& second)
    {
        return DDS_TrustAlgorithmRequirements_equals(&first, &second)
                == DDS_BOOLEAN_TRUE;
    }
};

class TrustAlgorithmRequirements;

} } // namespace topic::trust

namespace core {

template <>
struct native_type_traits<rti::topic::trust::TrustAlgorithmRequirements> {
    typedef rti::topic::trust::TrustAlgorithmRequirementsAdapter adapter_type;
    typedef DDS_TrustAlgorithmRequirements native_type;
};

}

namespace topic { namespace trust {

/**
 * @ingroup DDSBuiltInTopicCommonModule
 * @RTI_class_definition class TrustAlgorithmRequirements
 * @headerfile rti/topic/trust/TrustAlgorithmRequirements.hpp
 *
 * @brief @extension @st_value_type Type to describe Trust Plugins algorithm
 * requirements.
 *
 * @details \dref_details_TrustAlgorithmRequirements
 */
class OMG_DDS_API TrustAlgorithmRequirements : public rti::core::NativeValueType<TrustAlgorithmRequirements> {
public:
    RTI_NATIVE_VALUE_TYPE_DEFINE_DEFAULT_MOVE_OPERATIONS(TrustAlgorithmRequirements)

    typedef rti::core::NativeValueType<TrustAlgorithmRequirements> Base;
    /**
     * @brief Create an instance of TrustAlgorithmRequirements with default
     * values.
     *
     *  @details \dref_details_TrustAlgorithmRequirements_new
     */
    TrustAlgorithmRequirements() = default;

    TrustAlgorithmRequirements(
            const struct DDS_TrustAlgorithmRequirements& native_algo)
      : Base(native_algo)
    {
    }

    /**
     * @brief Get the Trust Plugins algorithms that an entity supports.
     *
     * @details \dref_details_TrustAlgorithmRequirements_supported_mask
     */
    uint32_t supported_mask() const
    {
        return native().supported_mask;
    }

    /**
     * @brief Get the Trust Plugins algorithms that an entity uses.
     *
     * @details \dref_details_TrustAlgorithmRequirements_required_mask
     */
    uint32_t required_mask() const
    {
        return native().required_mask;
    }

};

} } }

#endif // RTI_DDS_TOPIC_TRUST_ALGORITHM_REQUIREMENTS_HPP_
