/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)netio_receiver.h    generated by: makeheader    Sat Mar  9 09:38:15 2024
 *
 *		built from:	receiver.ifc
 */

#ifndef netio_receiver_h
#define netio_receiver_h


  #ifndef netio_dll_h
    #include "netio/netio_dll.h"
  #endif
  #ifndef netio_common_h
    #include "netio/netio_common.h"
  #endif
  #ifndef netio_cap_include_manager_h
    #include "netio_cap/netio_cap_manager.h"
  #endif

#ifdef __cplusplus
    extern "C" {
#endif

struct RTIClock;

struct REDABuffer;

struct REDAExclusiveArea;

struct REDAWorker;

struct REDAWeakReference;

struct REDADatabase;

struct RTIEventActiveObjectListener;

struct RTINetioConfigurator;

struct RTINetioReceiverProperty {
    /*e
      bitmap specifies which property the user is specifying actively.
      Those not indicated should be left as default.
      Boolean property such as whether blocking receive is to be used can
      be specified through just bitmap.

      @see RTI_NETIO_RECEIVER_PROPERTY_BIT_POLLED
      RTI_NETIO_RECEIVER_PROPERTY_BIT_TIMESTAMP
      RTI_NETIO_RECEIVER_PROPERTY_BIT_BOUND
    */
    int bitmap;

    /*e Control resource growth.  Helps avoid memory allocation. */
    struct REDAFastBufferPoolGrowthProperty resourceGrowth;

    /*e Control entryport growth.  Helps avoid memory allocation. */
    struct REDAFastBufferPoolGrowthProperty entryportGrowth;

    /*e
     * Print warning message when processing a received message operation time
     * exceeds processMessageTimeBasedLoggingThreshold.
     * See also processMessageTimeBasedLoggingExplicitPorts.
     * Default: INFINITE
     */
    RTINtpTime processMessageTimeBasedLoggingThreshold;
    /*e
     * Print warning message when processing a received message operation time
     * exceeds processMessageTimeBasedLoggingThreshold using a port in
     * processMessageTimeBasedLoggingExplicitPorts.
     *
     * Filter out the ports on which we do time-based logging.
     *
     * The ports will be separated by comma ','. They can be described using
     * regular expresion. For example "76*,1234".
     * Apply to all the ports if processMessageTimeBasedLoggingExplicitPorts is NULL.
     * Default: (NULL) all the ports.
     *
     * This string is:
     *  - Allocated in: RTINetioReceiverProperty_copy().
     *  - Released in RTINetioReceiver_delete().
     */
    char *processMessageTimeBasedLoggingExplicitPorts;
};


  #define RTI_NETIO_RECEIVER_PROPERTY_BIT_POLLED (0x1)


  #define RTI_NETIO_RECEIVER_PROPERTY_BIT_TIMESTAMP (0x2)


  #define RTI_NETIO_RECEIVER_PROPERTY_BIT_BOUND (0x4)


#define RTI_NETIO_RECEIVER_PROPERTY_DEFAULT \
{ \
    RTI_NETIO_RECEIVER_PROPERTY_BIT_TIMESTAMP \
        | RTI_NETIO_RECEIVER_PROPERTY_BIT_BOUND, /* bitmap */ \
    {16, REDA_FAST_BUFFER_POOL_UNLIMITED, REDA_FAST_BUFFER_POOL_UNLIMITED}, /* resourceGrowth */ \
    {8, REDA_FAST_BUFFER_POOL_UNLIMITED, REDA_FAST_BUFFER_POOL_UNLIMITED}, /* entryportGrowth */ \
    {RTI_NTP_TIME_SEC_MAX, RTI_NTP_TIME_FRAC_AUTO}, /* processMessageTimeBasedLoggingThreshold */ \
    NULL /* processMessageTimeBasedLoggingExplicitPorts */ \
}


#define RTINetioReceiverProperty_areThreadsBoundToRR(property__) \
  (((property__)->bitmap & RTI_NETIO_RECEIVER_PROPERTY_BIT_BOUND) != 0)

struct RTINetioReceiver;

extern RTINetioDllExport int
RTINetioReceiver_addEntryport(
        struct RTINetioReceiver *me,
        struct REDAWeakReference *entryportWRPtrOut,
        struct REDAWeakReference *receiveResourceWRPtrOut,
        const struct RTINetioLocatorInfo *entryportIn,
        struct REDAWorker *worker);

extern RTINetioDllExport int
RTINetioReceiver_removeEntryport(struct RTINetioReceiver *me,
		     const struct REDAWeakReference *entryportWRPtrIn,
		     struct REDAWorker *worker);

extern RTINetioDllExport int
RTINetioReceiver_checkEntryports(struct RTINetioReceiver *me,
        RTIBool *hasEntryPortsInUse,
        const struct REDAWeakReference *transportPluginWRPtrIn,
        struct REDAWorker *worker);


/*e
 * \ingroup RTINetioReceiverPropertyClass
 * \brief Indicates that data was read.
 */
#define RTI_NETIO_RECEIVER_RECEIVE_RETCODE_OK 1
/*e
 * \ingroup RTINetioReceiverPropertyClass
 * \brief Indicates that no data was read.
 */
#define RTI_NETIO_RECEIVER_RECEIVE_RETCODE_NO_DATA 0
/*e
 * \ingroup RTINetioReceiverPropertyClass
 * \brief Indicates that a generic error happened during the receive call.
 */
#define RTI_NETIO_RECEIVER_RECEIVE_RETCODE_UNBLOCK -1
/*e
 * \ingroup RTINetioReceiverPropertyClass
 * \brief Indicates that the bound receive resource was not found or that the
 * access to the resource failed. In this case, if there is a thread bound to
 * the receive resource, the RTINetioReceiver_receiveFast() caller needs to
 * destroy the thread.
 */
#define RTI_NETIO_RECEIVER_RECEIVE_RETCODE_NOT_FOUND -2
/*e
 * \ingroup RTINetioReceiverPropertyClass
 * \brief Indicates that the RTINetioReceiver_receiveFast() caller should call
 * to RTINetioReceiver_returnLoanAndDisownResource.
 */
#define RTI_NETIO_RECEIVER_RECEIVE_RETCODE_NEEDS_DISOWN -3

extern RTINetioDllExport int RTINetioReceiver_receiveFast(
        struct RTINetioReceiver *me,
        struct RTINetioMessage *messageOut,
        struct REDAWeakReference *resourceWRInOut,
        const struct REDABuffer *bufferIn,
        struct REDAWorker *worker);

extern RTINetioDllExport int
RTINetioReceiver_getRequiredThreadCount(struct RTINetioReceiver *me,
					struct REDAWorker *worker);

extern RTINetioDllExport
struct RTINetioReceiver *RTINetioReceiver_new(
        struct RTIClock *clockIn,
        struct REDADatabase *dbIn,
        struct RTIEventActiveObjectListener *listenerIn,
        struct RTINetioConfigurator *configuratorIn,
        const struct RTINetioReceiverProperty *propertyIn,
        struct RTINetioCapManager *netioCapManagerIn,
        struct REDAWorker *workerIn);

extern RTINetioDllExport RTIBool
RTINetioReceiver_preShutdownWakeup(struct RTINetioReceiver *me,
				   struct REDAWorker *worker);

extern RTINetioDllExport RTIBool
RTINetioReceiver_shutdown(struct RTINetioReceiver *me,
                          struct REDAWorker *worker);

extern RTINetioDllExport void
RTINetioReceiver_delete(struct RTINetioReceiver *me,
                        struct REDAWorker *worker);

extern RTINetioDllExport RTIBool RTINetioReceiver_bindResource(
        struct RTINetioReceiver *me,
        struct REDAWeakReference *resourceRecordWR,
        struct REDAWorker *receiveThreadWorker,
        struct REDAWorker *worker);

extern RTINetioDllExport void RTINetioReceiver_returnLoan(
        struct RTINetioReceiver *self,
        struct REDAWeakReference *resourceRecordWR,
        NDDS_Transport_Message_t *loanedMessage,
        struct REDAWorker *worker);

extern RTINetioDllExport void RTINetioReceiver_returnLoanAndDisownResource(
        struct RTINetioReceiver *self,
        struct REDAWeakReference *resourceRecordWR,
        NDDS_Transport_Message_t *loanedMessage,
        struct REDAWorker *worker);


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* netio_receiver_h */
