/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_context.h    generated by: makeheader    Sat Mar  9 09:37:08 2024
 *
 *		built from:	context.ifc
 */

#ifndef osapi_context_h
#define osapi_context_h


  #ifndef log_common_h
    #include "log/log_common.h"
  #endif
  #ifndef osapi_dll_h
    #include "osapi/osapi_dll.h"
  #endif
  #ifndef osapi_type_h
    #include "osapi/osapi_type.h"
  #endif
  #ifndef osapi_thread_h
    #include "osapi/osapi_thread.h"
  #endif
  #ifndef osapi_heap_h
    #include "osapi/osapi_heap.h"
  #endif
  #ifndef osapi_utility_h
    #include "osapi/osapi_utility.h"
  #endif
#ifdef __cplusplus
    extern "C" {
#endif


#define RTI_OSAPI_CONTEXT_STRING_SIZE_MAX 512


#define RTI_OSAPI_CONTEXT_STACK_SIZE_DEFAULT 32


#define RTI_OSAPI_ACTIVITY_CONTEXT_INDEX 0
#define RTI_OSAPI_HEAP_CONTEXT_INDEX 1


struct RTIOsapiContextEntry {
    void *entry;
    /* If the context is cached in expansion, expansionEnd points to end */
    char *expansionEnd;
    /*
     * Represent if an entry can be skipped from the string representation of
     * the context stack.
     *
     * We need to cache if the entry is skipped to optimize the generation of
     * the activity context string: we need to check previous entries skipped
     * state to make decisions about the new entries we add.
     *
     * For example in the activity context the output will change (we will skip
     * entries) based on the order and content of the entries:
     *  - For example similar activities and different resources:
     *    `R1|A1|R2|A1` -> `R1|A1|R2|A1`. The activities are similar but cannot
     *    be skipped because they belong to different resources.
     *  - For example similar activities and similar resources:
     *    `R1|A1|R1|A1` -> `R1|A1`. The activities and the resources are exactly
     *    the same, so both of them are skipped.
     */
    RTIBool skipped;
};


struct RTIOsapiContext {
    /* List of entries */
    struct RTIOsapiContextEntry *entries;
    RTI_UINT32 maxEntryCount;
    /* Position on the list */
    RTI_UINT32 top;
    /* Used to cache the context as a string */
    char *expansion;
    /* A bitmap used to store the categories associated with the context */
    RTILogCategoryMask category;
};

extern RTIOsapiDllExport
RTIBool RTIOsapiContextSupport_enable(RTIOsapiThreadTssFactory *tssFactory);

extern RTIOsapiDllExport
void RTIOsapiContextSupport_disable(void);

extern RTIOsapiDllExport
RTIBool RTIOsapiContextSupport_assertContextTss(
        RTI_UINT32 contextIndex,
        RTIBool addContextTssToGlobalList,
        RTI_UINT32 maxEntryCount,
        RTIBool allowContextExpansion);

extern RTIOsapiDllExport
void RTIOsapiContextSupport_finalizeContextTss(void);

extern RTIOsapiDllExport
struct RTIOsapiContext * RTIOsapiContextSupport_getContext(
        RTI_UINT32 contextIndex);

extern RTIOsapiDllExport
void RTIOsapiContext_enter(
        struct RTIOsapiContext *self,
        RTI_UINT32 contextIndex,
        void *entry);

extern RTIOsapiDllExport
void RTIOsapiContext_enterPair(
        struct RTIOsapiContext *self,
        RTI_UINT32 contextIndex,
        void *entry1,
        void *entry2);

extern RTIOsapiDllExport
void RTIOsapiContext_leave(
        struct RTIOsapiContext *self,
        RTI_UINT32 contextIndex);

extern RTIOsapiDllExport
void RTIOsapiContext_leaveGroup(
        struct RTIOsapiContext *self,
        RTI_UINT32 contextIndex,
        RTI_UINT32 groupSize);

extern RTIOsapiDllExport
RTI_UINT32 RTIOsapiContext_getDepth(RTI_UINT32 contextIndex);

extern RTIOsapiDllExport
void RTIOsapiContext_getCategory(
        struct RTIOsapiContext *self,
        RTILogCategoryMask *category,
        RTI_UINT32 contextIndex);

extern RTIOsapiDllExport
void RTIOsapiContext_setCategory(
        struct RTIOsapiContext *self,
        RTI_UINT32 contextIndex,
        RTILogCategoryMask category);

extern RTIOsapiDllExport
void RTIOsapiContext_addCategory(
        struct RTIOsapiContext *self,
        RTI_UINT32 contextIndex,
        RTILogCategoryMask category);

extern RTIOsapiDllExport
void RTIOsapiContext_addAndCacheCategory(
        struct RTIOsapiContext *self,
        RTILogCategoryMask *cachedCategory,
        RTI_UINT32 contextIndex,
        RTILogCategoryMask category);

extern RTIOsapiDllExport
void RTIOsapiContext_resetCategory(
        struct RTIOsapiContext *self,
        RTI_UINT32 contextIndex,
        RTILogCategoryMask category);

extern RTIOsapiDllExport
void RTIOsapiContextScope_begin(void);

extern RTIOsapiDllExport
void RTIOsapiContextScope_end(void);


#ifdef __cplusplus
    }	/* extern "C" */
#endif
#include "osapi/osapi_context_impl.h"

#endif /* osapi_context_h */
