/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_context_impl.h    generated by: makeheader    Sat Mar  9 09:37:08 2024
 *
 *		built from:	context_impl.ifc
 */

#ifndef osapi_context_impl_h
#define osapi_context_impl_h


#ifdef __cplusplus
    extern "C" {
#endif



/*i
 * \ingroup RTIOsapiContextSupport
 * It is a counter which specify how many times the context support has been
 * initialized.
 * It is used to support multiple users calling RTIOsapiContextSupport_enable /
 * RTIOsapiContextSupport_disable, while ensuring we only enable/disable
 * the context once.
 */
extern RTIOsapiDllVariable RTI_UINT32
        RTIOsapiContextSupport_g_tssInitializedRefCount;


/*i
 * \ingroup RTIOsapiContextSupport
 * The thread-specific storage (TSS) data key visible to all threads in the
 * process.
 * The valid range for keys is [0, RTI_UINT32_MAX]. We use an RTI_INT64 because
 * we use the -1 value for flagging that this global variable has not been
 * initialized yet (RTI_OSAPI_CONTEXT_SUPPORT_TSS_KEY_INVALID).
 * RTI_UINT32_MAX always fits inside an RTI_INT64. Therefore, the casting is
 * safe as long as you control the -1 case.
 */
extern RTIOsapiDllVariable RTI_INT64 RTIOsapiContextSupport_g_tssKey;



/*i
 * \ingroup RTIOsapiContextSupport
 * Value used to indicate that RTIOsapiContextSupport_g_tssKey doesn't have a
 * valid data key.
 */
#define RTI_OSAPI_CONTEXT_SUPPORT_TSS_KEY_INVALID -1

/*i
 * \ingroup RTIOsapiContextSupport
 * Checks whether key__ is or not a valid TSS data key.
 */
#define RTIOsapiContextSupport_tssKeyIsValid(key__) \
  ((key__) != RTI_OSAPI_CONTEXT_SUPPORT_TSS_KEY_INVALID)


/*i \ingroup RTIOsapiContext
 * Number of type of contexts.
 * Instantiate different specializations of the context:
 * - Activity context: it will describe the activity that a resource is doing.
 * - Heap context: it will represent: If the allocation is from the pool and in
 * that case the buffer size and the buffer count. The topic and the function name.
 *- In the future, we can have more potential context families for instance
 * network capture context.
 */

#define RTI_OSAPI_NUMBER_CONTEXTS 3


/*i \ingroup RTIOsapiContextSupport
 * Each node contains the contexts of the thread.
 */

struct RTIOsapiContextSupportNode {
    struct RTIOsapiInlineListNode node;
    struct RTIOsapiContext *contexts[RTI_OSAPI_NUMBER_CONTEXTS];
};


/*
 * @brief It is a macro to get the context of the caller thread, specified by
 * the Id.
 * It is used in the critical path and in the function:
 *      RTIOsapiContextSupport_getContext
 *
 * @param contextIndex \b In. The index of the context.
 * @param the context \b Out. The context.
 */
#define RTIOsapiContextSupport_getContextI(__contextIndex, __context) \
  { \
    RTI_INT64 localTssKey__ = RTIOsapiContextSupport_g_tssKey; \
    if (RTIOsapiContextSupport_tssKeyIsValid(localTssKey__)) { \
      struct RTIOsapiContextSupportNode *__node = \
              (struct RTIOsapiContextSupportNode *) RTIOsapiThread_getTss( \
                      (RTI_UINT32) localTssKey__); \
      if (__node != NULL) { \
        __context = __node->contexts[__contextIndex]; \
      } \
    } \
  }

#define RTIOsapiContextScope_begin() \
    {

#define RTIOsapiContextScope_end() \
    }

#define RTIOsapiContext_enterMacro( \
        self__, \
        contextIndex__, \
        entry__) \
{ \
    struct RTIOsapiContext *context__ = (self__); \
    if ((context__) == NULL) { \
        RTIOsapiContextSupport_getContextI((contextIndex__), (context__)); \
    } \
    if ((context__) != NULL) { \
        if ((context__)->top < (context__)->maxEntryCount) { \
            /* Enter entry */ \
            (context__)->entries[(context__)->top].entry = (entry__); \
            (context__)->entries[(context__)->top].expansionEnd = NULL; /* no cached expansion */ \
            (context__)->entries[(context__)->top].skipped = RTI_FALSE; \
        } \
        (context__)->top++; \
    } \
}

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_enter RTIOsapiContext_enterMacro
#endif

#define RTIOsapiContext_leaveMacro( \
        self__, \
        contextIndex__) \
{ \
    struct RTIOsapiContext *context__ = (self__); \
    if (context__ == NULL) { \
        RTIOsapiContextSupport_getContextI((contextIndex__), (context__)); \
    } \
    if ((context__) != NULL) { \
        --(context__)->top; \
    } \
}

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_leave RTIOsapiContext_leaveMacro
#endif

#define RTIOsapiContext_enterPairMacro( \
        __self, \
        __contextIndex, \
        __entry1, \
        __entry2) \
{ \
    struct RTIOsapiContext *__context = (__self); \
    if ((__context) == NULL) { \
        RTIOsapiContextSupport_getContextI((__contextIndex), (__context)); \
    } \
    if ((__context) != NULL) { \
        if ((__context)->top + 2 <= (__context)->maxEntryCount) { \
            struct RTIOsapiContextEntry *__entry = \
                    &(__context)->entries[(__context)->top]; \
            /* Enter entry 1 */ \
            __entry[0].entry = (__entry1); \
            __entry[0].expansionEnd = NULL; /* no cached expansion */ \
            __entry[0].skipped = RTI_FALSE; \
            /* Enter entry 2 */ \
            __entry[1].entry = (__entry2); \
            __entry[1].expansionEnd = NULL; /* no cached expansion */ \
            __entry[1].skipped = RTI_FALSE; \
        } \
        (__context)->top += 2; \
    } \
}

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_enterPair RTIOsapiContext_enterPairMacro
#endif

#define RTIOsapiContext_leaveGroupMacro( \
        __self, \
        __contextIndex, \
        __groupSize) \
{ \
    if ((__groupSize) > 0) { \
        struct RTIOsapiContext *__context = (__self); \
        if (__context == NULL) { \
            RTIOsapiContextSupport_getContextI((__contextIndex), (__context)); \
        } \
        if ((__context) != NULL) { \
            if ((__groupSize) > (__context)->top) { \
                (__context)->top = 0; \
            } else { \
                (__context)->top -= (__groupSize); \
            } \
        } \
    } \
}

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_leaveGroup RTIOsapiContext_leaveGroupMacro
#endif

#define RTIOsapiContext_getCategoryMacro( \
        self__, \
        category__, \
        contextIndex__) \
    { \
        struct RTIOsapiContext *context__ = (self__); \
        if (context__ == NULL) { \
            RTIOsapiContextSupport_getContextI((contextIndex__), (context__)); \
         } \
        if ((context__) != NULL) { \
            *(category__) = (context__)->category; \
        } \
    }

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_getCategory RTIOsapiContext_getCategoryMacro
#endif

#define RTIOsapiContext_setCategoryMacro( \
        self__, \
        contextIndex__, \
        category__) \
    { \
        struct RTIOsapiContext *context__ = (self__); \
        if (context__ == NULL) {\
            RTIOsapiContextSupport_getContextI((contextIndex__), (context__)); \
         } \
        if ((context__) != NULL) { \
            (context__)->category = (category__);\
        } \
    }

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_setCategory RTIOsapiContext_setCategoryMacro
#endif

#define RTIOsapiContext_addCategoryMacro( \
        self__, \
        contextIndex__, \
        category__) \
    { \
        struct RTIOsapiContext *context__ = (self__); \
        if( context__ == NULL) { \
            RTIOsapiContextSupport_getContextI((contextIndex__), (context__)); \
         } \
        if ((context__) != NULL) { \
            (context__)->category |= (category__);\
        } \
    }

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_addCategory RTIOsapiContext_addCategoryMacro
#endif

#define RTIOsapiContext_resetCategoryMacro( \
        self__, \
        contextIndex__, \
        category__) \
    { \
        struct RTIOsapiContext *context__ = (self__); \
        if (context__ == NULL) {\
            RTIOsapiContextSupport_getContextI((contextIndex__), (context__)); \
         } \
        if ((context__) != NULL) { \
            (context__)->category &= ~(category__);\
        } \
    }

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_resetCategory RTIOsapiContext_resetCategoryMacro
#endif

#define RTIOsapiContext_addAndCacheCategoryMacro( \
        self__, \
        cachedCategory__, \
        contextIndex__, \
        category__) \
    { \
        struct RTIOsapiContext *context__ = (self__); \
        if (context__ == NULL) {\
            RTIOsapiContextSupport_getContextI((contextIndex__), (context__)); \
         } \
        if ((context__) != NULL) { \
            *(cachedCategory__) = (context__)->category; \
            (context__)->category |= (category__);\
        } \
    }

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiContext_addAndCacheCategory RTIOsapiContext_addAndCacheCategoryMacro
#endif


#ifdef __cplusplus
    }	/* extern "C" */
#endif


#endif /* osapi_context_impl_h */
