/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)reda_sequenceNumber_impl.h    generated by: makeheader    Sat Mar  9 09:37:42 2024
 *
 *		built from:	sequenceNumber_impl.ifc
 */

#ifndef reda_sequenceNumber_impl_h
#define reda_sequenceNumber_impl_h


#ifdef __cplusplus
    extern "C" {
#endif


  #define REDASequenceNumber_expand(__sequenceNumber) \
    (__sequenceNumber)->high, \
    (__sequenceNumber)->low

  #define REDASequenceNumber_setZero(sequenceNumber) \
    (sequenceNumber)->high = 0; \
    (sequenceNumber)->low = 0

  #define REDASequenceNumber_isZero(sequenceNumber) \
    ((sequenceNumber)->high == 0 && \
     (sequenceNumber)->low == 0)

  #define REDASequenceNumber_setUnknown(sequenceNumber) \
    (sequenceNumber)->high = REDA_SEQUENCE_NUMBER_HIGH_UNKNOWN; \
    (sequenceNumber)->low = REDA_SEQUENCE_NUMBER_LOW_UNKNOWN

  #define REDASequenceNumber_isUnknown(sequenceNumber) \
    ((sequenceNumber)->high == REDA_SEQUENCE_NUMBER_HIGH_UNKNOWN && \
     (sequenceNumber)->low == REDA_SEQUENCE_NUMBER_LOW_UNKNOWN)

  #define REDASequenceNumber_isAuto(sequenceNumber) \
    ((sequenceNumber)->high == REDA_SEQUENCE_NUMBER_HIGH_AUTO && \
     (sequenceNumber)->low == REDA_SEQUENCE_NUMBER_LOW_AUTO)

  #define REDASequenceNumber_isMaximum(sequenceNumber) \
    ((sequenceNumber)->high == REDA_SEQUENCE_NUMBER_HIGH_MAX && \
     (sequenceNumber)->low == REDA_SEQUENCE_NUMBER_LOW_MAX)

  #define REDASequenceNumber_setMaximum(sequenceNumber) \
    (sequenceNumber)->high = REDA_SEQUENCE_NUMBER_HIGH_MAX; \
    (sequenceNumber)->low = REDA_SEQUENCE_NUMBER_LOW_MAX

  #define REDASequenceNumber_compare(sequenceNumber1, sequenceNumber2) \
    ((((sequenceNumber1)->high) > ((sequenceNumber2)->high)) ? 1 : \
     ((((sequenceNumber1)->high) < ((sequenceNumber2)->high)) ? -1 : \
      ((((sequenceNumber1)->low) > ((sequenceNumber2)->low)) ? 1 : \
       ((((sequenceNumber1)->low) < ((sequenceNumber2)->low)) ? -1 : 0))))

  #define REDASequenceNumber_add(answer, sn1, sn2) \
  { \
    (answer)->high = (sn1)->high + (sn2)->high; \
    (answer)->low = (sn1)->low + (sn2)->low; \
    if (((answer)->low < (sn1)->low) || ((answer)->low < (sn2)->low)) { \
        ++(answer)->high; \
    } \
  }

  #define REDASequenceNumber_subtract(answer, sn1, sn2) \
  { \
   (answer)->high  = (sn1)->high - (sn2)->high; \
   (answer)->low = (sn1)->low - (sn2)->low; \
   if ((answer)->low > (sn1)->low) { \
       --(answer)->high; \
    } \
  }

  #define REDASequenceNumber_plusplus(sequenceNumber) \
    ((++(sequenceNumber)->low) == 0) ? ++(sequenceNumber)->high : 0

  #define REDASequenceNumber_minusminus(sequenceNumber) \
  { \
    register RTI_UINT32 originalLow = (sequenceNumber)->low; \
    --(sequenceNumber)->low; \
    if (((sequenceNumber)->low > originalLow)) { --(sequenceNumber)->high; } \
  }

  #define REDASequenceNumber_increment(answer, sequenceNumber) \
  { \
    register RTI_UINT32 currentLow = (answer)->low; \
    (answer)->high += (sequenceNumber)->high; \
    (answer)->low += (sequenceNumber)->low; \
    if (((answer)->low < (sequenceNumber)->low) || \
        ((answer)->low < currentLow)) { \
        ++(answer)->high; \
    } \
  }

  #define REDASequenceNumber_decrement(answer, sequenceNumber) \
  { \
    register RTI_UINT32 currentLow = (answer)->low; \
    (answer)->high -= (sequenceNumber)->high; \
    (answer)->low -= (sequenceNumber)->low; \
    if (((answer)->low > currentLow)) { --(answer)->high; } \
  }

  #define REDASequenceNumber_max(answer, sn1, sn2) \
  { \
    if (REDASequenceNumber_compare((sn1), (sn2)) > 0) { \
      (answer)->high = (sn1)->high; (answer)->low = (sn1)->low; \
    } else { \
      (answer)->high = (sn2)->high; (answer)->low = (sn2)->low; \
    } \
  }

  #define REDASequenceNumber_min(answer, sn1, sn2) \
  { \
    if (REDASequenceNumber_compare((sn1), (sn2)) < 0) { \
      (answer)->high = (sn1)->high; (answer)->low = (sn1)->low; \
    } else { \
      (answer)->high = (sn2)->high; (answer)->low = (sn2)->low; \
    } \
  }

  #define REDASequenceNumber_toDouble(sn) \
  ( (double)((sn)->low) + ((double)((sn)->high) * 4294967296.0) ) 

#ifdef RTI_ENDIAN_LITTLE
#define REDASequenceNumber_fromLongLong(sn, src) \
    RTIOsapiUtility_assignValueFromDereference( \
            (sn)->low, \
            src, \
            unsigned int); \
    RTIOsapiUtility_assignValueFromDereference( \
            (sn)->high, \
            (int *) src + 1, \
            int);
#define REDASequenceNumber_toLongLong(dst, sn) \
    RTIOsapiUtility_assignDereferenceFromValue( \
            dst, \
            (sn)->low, \
            unsigned int); \
    RTIOsapiUtility_assignDereferenceFromValue( \
            (int *) dst + 1, \
            (sn)->high, \
            int);
#else
#define REDASequenceNumber_fromLongLong(sn, src) \
    RTIOsapiUtility_assignValueFromDereference( \
            (sn)->high, \
            src, \
            unsigned int); \
    RTIOsapiUtility_assignValueFromDereference( \
            (sn)->low, \
            (int *) src + 1, \
            int);
#define REDASequenceNumber_toLongLong(dst, sn) \
    RTIOsapiUtility_assignDereferenceFromValue( \
            dst, \
            (sn)->high, \
            unsigned int); \
    RTIOsapiUtility_assignDereferenceFromValue( \
            (int *) dst + 1, \
            (sn)->low, \
            int);
#endif




  #define REDASequenceNumberIntervalList_isInitialized(self) \
  (((self)->_list_init == REDA_SEQUENCE_NUMBER_INTERVAL_LIST_MAGIC_NUMBER)?RTI_TRUE:RTI_FALSE)

  #define REDASequenceNumberIntervalList_assertAndUpdateSequenceNumberWithUserData( \
          self__, \
          sn__, \
          userData__, \
          userDataExpirationTime__) \
  ( \
      REDASequenceNumberIntervalList_deleteSequenceNumber(self__, sn__) \
              && REDASequenceNumberIntervalList_assertSequenceNumberWithUserData( \
              self__, \
              NULL, /* alreadyExists */ \
              sn__, \
              userData__, \
              userDataExpirationTime__) \
  )

  #define REDASequenceNumberInterval_getSampleCount(self) \
  (RTI_UINT64)(((((RTI_UINT64)((self)->lastSn.high - (self)->firstSn.high) << 32) - (RTI_UINT64)((self)->lastSn.high - (self)->firstSn.high)) + (((self)->lastSn.low - (self)->firstSn.low) + 1)))



#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* reda_sequenceNumber_impl_h */
