/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)reda_worker_impl.h    generated by: makeheader    Sat Mar  9 09:37:43 2024
 *
 *		built from:	worker_impl.ifc
 */

#ifndef reda_worker_impl_h
#define reda_worker_impl_h


#include "reda/reda_inlineList.h"
#include "osapi/osapi_thread.h"

#if defined(PURIFY_TEST)
  #define RTI_DEBUG_ENABLE_EA_EXCLUSION_REQUIREMENTS
#endif

#ifdef RTI_DEBUG_ENABLE_EA_EXCLUSION_REQUIREMENTS
  #include "reda/reda_log.h"
  #include "reda/reda_skiplist.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif



#define REDA_WORKER_OBJECT_PER_WORKER_MAX_BUCKETS 8

#define REDA_TABLE_EXCLUSIVE_AREA_LEVEL 10
#define REDA_SUBSCRIBER_EXCLUSIVE_AREA_LEVEL 20
#define REDA_PUBLISHER_EXCLUSIVE_AREA_LEVEL 30
#define REDA_BLOCKING_EXCLUSIVE_AREA_LEVEL 40
#define REDA_RECORD_EXCLUSIVE_AREA_LEVEL 40
#define REDA_REMOTE_EXCLUSIVE_AREA_LEVEL 40
#define REDA_CONDITION_EXCLUSIVE_AREA_LEVEL 40
#define REDA_NETIO_RECEIVER_TABLE_EXCLUSIVE_AREA_LEVEL 45
#define REDA_DATABASE_EXCLUSIVE_AREA_LEVEL 50
#define REDA_WAITSET_EXCLUSIVE_AREA_LEVEL 70

#define REDA_EXCLUSIVE_AREA_UNNAMED_NAME "UNNAMED_EA"
#define REDA_TABLE_EXCLUSIVE_AREA_NAME "DP_TABLE_EA"
#define REDA_SUBSCRIBER_EXCLUSIVE_AREA_NAME "SUBSCRIBER_EA"
#define REDA_PUBLISHER_EXCLUSIVE_AREA_NAME "PUBLISHER_EA"
#define REDA_BLOCKING_EXCLUSIVE_AREA_NAME "DP_BLOCKING_EA"
#define REDA_RECORD_EXCLUSIVE_AREA_NAME "DP_RECORD_EA"
#define REDA_REMOTE_EXCLUSIVE_AREA_NAME "DP_REMOTE_EA"
#define REDA_CONDITION_EXCLUSIVE_AREA_NAME "DDS_CONDITION_EA"
#define REDA_NETIO_RECEIVER_TABLE_EXCLUSIVE_AREA_NAME "NETIO_RECEIVER_EA"
#define REDA_DATABASE_EXCLUSIVE_AREA_NAME "DP_DB_EA"
#define REDA_WAITSET_EXCLUSIVE_AREA_NAME "DDS_WAITSET_EA"

struct REDAWorkerForbiddenEANameNode {
    struct REDAInlineListNode node;
    const char *eaName;
};

struct REDAWorkerForbiddenEALevelNode {
    struct REDAInlineListNode node;
    int eaLevel;
};


struct REDAWorker {
    /*i node in list inside WorkerFactory */
    struct REDAInlineListNode _node;
    /*i optional name (for informational purposes only) */
    char *_name;
    /*i The manager */
    struct REDAWorkerFactory  *_manager;
    /*i An array pointing to worker-specific objects. */
    void **_workerSpecificObject[REDA_WORKER_OBJECT_PER_WORKER_MAX_BUCKETS];
    /*i A linked list of the EAs this worker is currently in */
    struct REDAInlineList _inEA;
    /*i EA with highest level */
    const struct REDAExclusiveArea *_inEaMaxLevelPtr;
    /*i Activity Context of the thread associte to the Worker */
    struct RTIOsapiContext *_activityContext;
#ifdef RTI_DEBUG_ENABLE_EA_EXCLUSION_REQUIREMENTS
    /*i List of forbidden EAs for this worker */
    struct REDAInlineList _forbiddenEaNameList;
    /*i List of forbidden EA levels (equal or lower) for this worker */
    struct REDAInlineList _forbiddenEaLevelList;
#endif /* RTI_DEBUG_ENABLE_EA_EXCLUSION_REQUIREMENTS */
};


struct REDAObjectPerWorker {
    /*i optional name (for informational purposes only) */
    const char *_name;
    /*i the index of this object in the first level worker's array */
    int _objectBucketIndex;
    /*i the index of this object in the second level worker's array */
    int _objectIndexInBucket;
    /*i the constructor */
    REDAObjectPerWorkerConstructor _constructor;
    /*i the constructor param */
    void *_constructorParameter;
    /*i the destructor */
    REDAObjectPerWorkerDestructor _destructor;
    /*i the destructor param */
    void *_destructorParameter;
};


struct REDAWorkerPerThread {
    /*i @brief Factory used to create workers */
    struct REDAWorkerFactory * _workerFactory;
    RTIBool _ownsFactory;    
    /*i
     * @brief   Thread-specific storage key for a worker.
     */
    RTI_UINT32                 _workerTssKey;
    /*i
     * @brief Used to generate thread-specific-storage keys
     */
    RTIOsapiThreadTssFactory * _tssFactory;
    RTIBool _ownsTss;
};



/* ----------------------------------------------------------------- */
#define REDAWorker_getName(worker__) (worker__)->_name

/* ----------------------------------------------------------------- */
#define REDAWorker_getActivityContext(worker) \
    (worker) == NULL \
            ? NULL  \
            : ((struct REDAWorker *) (worker))->_activityContext
/* ----------------------------------------------------------------- */
#define REDAWorker_getFactory(worker) worker->_manager

/* ----------------------------------------------------------------- */
#define REDAWorker_getObjectMacro(worker, objectPerWorker)                  \
    ((worker)->_workerSpecificObject[(objectPerWorker)->_objectBucketIndex] \
                                    [(objectPerWorker)->_objectIndexInBucket])

/* ----------------------------------------------------------------- */
#define REDAWorker_assertObjectMacro(worker, objectPerWorker)           \
    ((REDAWorker_getObjectMacro((worker), (objectPerWorker)) != NULL)   \
             ? (REDAWorker_getObjectMacro((worker), (objectPerWorker))) \
             : ((worker)->_workerSpecificObject                         \
                        [(objectPerWorker)->_objectBucketIndex]         \
                        [(objectPerWorker)->_objectIndexInBucket] =     \
                   (*((objectPerWorker)->_constructor)) ((objectPerWorker)->_constructorParameter, worker)))

/* ----------------------------------------------------------------- */
#define REDAWorker_destroyObjectMacro(                                      \
        worker,                                                             \
        objectPerWorker,                                                    \
        callingThreadWorker)                                                \
    (*((objectPerWorker)->_destructor))(                                    \
            REDAWorker_getObjectMacro((worker), (objectPerWorker)),         \
            (objectPerWorker)->_destructorParameter,                        \
            callingThreadWorker);                                           \
    ((worker)->_workerSpecificObject[(objectPerWorker)->_objectBucketIndex] \
                                    [(objectPerWorker)                      \
                                             ->_objectIndexInBucket]) = NULL

/* ----------------------------------------------------------------- */
#ifndef RTI_PRECONDITION_TEST
  #define REDAWorker_getObject     REDAWorker_getObjectMacro
  #define REDAWorker_assertObject  REDAWorker_assertObjectMacro
  #define REDAWorker_destroyObject REDAWorker_destroyObjectMacro
#endif /* !RTI_PRECONDITION_TEST */

#define REDAWorkerFactory_destroyWorker(__workerFactory, __worker) \
    REDAWorkerFactory_destroyWorkerEx((__workerFactory), (__worker), (__worker))

#define REDAWorkerFactory_createExclusiveArea( \
        __workerFactory, \
        __level) \
    REDAWorkerFactory_createExclusiveAreaEx( \
            (__workerFactory), \
            (__level), \
            REDA_EXCLUSIVE_AREA_UNNAMED_NAME)




/*e
 * @brief Checks if worker is in any EA.
 * @param worker \b In. Thread worker.
 * @return RTI_TRUE if worker is any EA, RTI_FALSE otherwise.
 */
#define REDAWorker_isInAnyExclusiveArea(worker__) \
  (REDAInlineList_getSize(&(worker__)->_inEA) > 0)

#ifdef RTI_DEBUG_ENABLE_EA_EXCLUSION_REQUIREMENTS

/*e
 * @brief Initializes an Exclusive Area Requirement Scope.
 * @param maxForbiddenEANames__ \b In. Max number of name-based exclusion
 * requirements allowed for this scope.
 * @param maxForbiddenEALevels__ \b In. Max number of level-based exclusion
 * requirements allowed for this scope.
 * @pre REDAWorkerExclusiveAreaRequirementsScope_end should be called
 * after this macro and before the current scope ends.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_begin( \
          maxForbiddenEANames__, \
          maxForbiddenEALevels__) \
    { \
      struct REDAWorkerForbiddenEANameNode \
              exclusionAreaByName__[maxForbiddenEANames__] = { 0 }; \
      unsigned int exclusionAreaByNameCounter__ = 0; \
      struct REDAWorkerForbiddenEALevelNode \
              exclusionAreaByLevel__[maxForbiddenEALevels__] = { 0 }; \
      unsigned int exclusionAreaByLevelCounter__ = 0

/*i
 * @brief Reports an error occurred in a EA requirements scope as a
 * precondition.
 * @param ... \b In. A parameterized string and optionally list of params of
 * the parameterized string.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_reportError(...) \
    if (RTILog_g_detectPrecondition) { \
        RTILog_g_preconditionDetected = 1; \
    } \
    RTIOsapiUtility_unusedReturnValue( \
            RTIOsapiHeap_mallocWithoutHeapHeader(1), \
            void *); \
    RTILogParamString_print( \
            RTI_LOG_PRINT_FORMAT_MASK_ALL, \
            RTI_LOG_BIT_FATAL_ERROR, \
            MODULE_REDA, \
            __VA_ARGS__)

/*e
 * @brief Enters an Exclusive Area name exclusion requirement to a worker. This
 * will cause the EA with that name to be forbidden for the worker. EA
 * exclusions can be nested following a LIFO basis.
 * @param exclusiveAreaName__ \b In. EA name to be forbidden.
 * @param checkRequirementIsCurrentlyMet__ \b In. Check if the restriction is
 * currently met before adding it (aka check if the worker is already in an EA
 * with name exclusiveAreaName__).
 * @param worker__ \b In. Thread worker.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_enterExclusionByName( \
          exclusiveAreaName__, \
          checkRequirementIsCurrentlyMet__, \
          worker__) \
    if (checkRequirementIsCurrentlyMet__ \
        && REDAWorker_isInExclusiveArea(worker, exclusiveAreaName__)) { \
      REDAWorkerExclusiveAreaRequirementsScope_reportError( \
              "%s cannot be accessed within %s EA\n", \
              RTI_FUNCTION_NAME, \
              exclusiveAreaName__); \
    } \
    REDAInlineListNode_init( \
            &exclusionAreaByName__[exclusionAreaByNameCounter__].node); \
    exclusionAreaByName__[exclusionAreaByNameCounter__].eaName = \
            exclusiveAreaName__; \
    REDAInlineList_addNodeToBackEA( \
            &worker__->_forbiddenEaNameList, \
            &exclusionAreaByName__[exclusionAreaByNameCounter__].node); \
    exclusionAreaByNameCounter__++

/*e
 * @brief Leaves an EA name exclusion requirement in a worker. They work in a
 * LIFO basis, so last EA forbidden by name will not be forbidden anymore.
 * @param worker__ \b In. Thread worker.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_leaveExclusionByName( \
          worker__) \
    exclusionAreaByNameCounter__--; \
    REDAInlineList_removeNodeEA( \
            &worker__->_forbiddenEaNameList, \
            &exclusionAreaByName__[exclusionAreaByNameCounter__].node)

/*e
 * @brief Enters an EA level exclusion requirement to a worker. This will
 * cause EAs with lower level or equal to be forbidden for the worker. EA
 * exclusion areas can be nested following a LIFO basis.
 * @param exclusiveAreaLevel__ \b In. EA level to be forbidden.
 * @param checkRequirementIsCurrentlyMet__ \b In. Check if the restriction is
 * currently met before adding it (aka check if the worker is already in an EA
 * with level lower or equal than exclusiveAreaLevel__).
 * @param worker__ \b In. Thread worker.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_enterExclusionByLevel( \
          exclusiveAreaLevel__, \
          checkRequirementIsCurrentlyMet__, \
          worker__) \
    if (checkRequirementIsCurrentlyMet__ \
        && REDAWorker_isInExclusiveAreaLevelLowerOrEqualThan( \
                worker, \
                exclusiveAreaLevel__)) { \
      REDAWorkerExclusiveAreaRequirementsScope_reportError( \
              "%s cannot be accessed within an EA with a level lower " \
              "or equal to %d\n", \
              RTI_FUNCTION_NAME, \
              exclusiveAreaLevel__); \
    } \
    REDAInlineListNode_init( \
            &exclusionAreaByLevel__[exclusionAreaByLevelCounter__].node); \
    exclusionAreaByLevel__[exclusionAreaByLevelCounter__].eaLevel = \
            exclusiveAreaLevel__; \
    REDAInlineList_addNodeToBackEA( \
            &worker__->_forbiddenEaLevelList, \
            &exclusionAreaByLevel__[exclusionAreaByLevelCounter__].node); \
    exclusionAreaByLevelCounter__++

/*e
 * @brief Leaves an EA level exclusion requirement in a worker. They work in a
 * LIFO basis, so last forbidden EA level will not be forbidden anymore.
 * @param worker__ \b In. Thread worker.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_leaveExclusionByLevel( \
          worker__) \
    exclusionAreaByLevelCounter__--; \
    REDAInlineList_removeNodeEA( \
            &worker__->_forbiddenEaLevelList, \
            &exclusionAreaByLevel__[exclusionAreaByLevelCounter__].node)

/*e
 * @brief Finalizes an Exclusive Area Requirement Scope.
 * @pre REDAWorkerExclusiveAreaRequirementsScope_begin should be called
 * before this macro, within the same scope.
 * @pre All EA exclusion areas in this scope should be finished before calling
 * this.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_end() \
    /* Avoid unused variables warnings */ \
    RTIOsapiUtility_unusedParameter(exclusionAreaByName__[0]); \
    RTIOsapiUtility_unusedParameter(exclusionAreaByLevel__[0]); \
    if (exclusionAreaByNameCounter__ > 0) { \
      REDAWorkerExclusiveAreaRequirementsScope_reportError( \
              "%s: By name exclusive area exclusion area list " \
              "is not empty, it still contains %u elements\n", \
              RTI_FUNCTION_NAME, \
              exclusionAreaByNameCounter__); \
    } \
    if (exclusionAreaByLevelCounter__ > 0) { \
      REDAWorkerExclusiveAreaRequirementsScope_reportError( \
              "%s: By level exclusive area exclusion area list " \
              "is not empty, it still contains %u elements\n", \
              RTI_FUNCTION_NAME, \
              exclusionAreaByLevelCounter__); \
    } \
    } \
    (void) 0

/*e
 * @brief Initializes an Exclusive Area Requirement Scope and enters an EA name
 * exclusion requirement to a worker.
 * @param exclusiveAreaName__ \b In. EA name to be forbidden.
 * @param worker__ \b In. Thread worker.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_startExclusionByName( \
          exclusiveAreaName__, \
          skipCondition__, \
          worker__) \
    REDAWorkerExclusiveAreaRequirementsScope_begin(1, 1); \
    if (!(skipCondition__)) { \
      REDAWorkerExclusiveAreaRequirementsScope_enterExclusionByName( \
              exclusiveAreaName__, \
              RTI_TRUE /* checkRequirementIsCurrentlyMet */, \
              worker__); \
    } \
    (void) 0

/*e
 * @brief Leaves all EA name exclusion requirements in a worker and finalizes
 * the Exclusive Area Requirement Scope.
 * @param worker__ \b In. Thread worker.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_finishExclusionByName( \
          worker__) \
    while (exclusionAreaByNameCounter__ > 0) { \
        REDAWorkerExclusiveAreaRequirementsScope_leaveExclusionByName( \
                worker__); \
    } \
    REDAWorkerExclusiveAreaRequirementsScope_end()

/*e
 * @brief Initializes an Exclusive Area Requirement Scope and enters an EA level
 * exclusion requirement to a worker.
 * @param exclusiveAreaLevel__ \b In. EA level to be forbidden.
 * @param skipCondition__ \b In. If RTI_TRUE, this API is a NOOP.
 * @param worker__ \b In. Thread worker.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_startExclusionByLevel( \
          exclusiveAreaLevel__, \
          skipCondition__, \
          worker__) \
    REDAWorkerExclusiveAreaRequirementsScope_begin(1, 1); \
    if (!(skipCondition__)) { \
      REDAWorkerExclusiveAreaRequirementsScope_enterExclusionByLevel( \
              exclusiveAreaLevel__, \
              RTI_TRUE /* checkRequirementIsCurrentlyMet */, \
              worker__); \
    } \
    (void) 0

/*e
 * @brief Leaves all EA level exclusion requirements in a worker and finalizes
 * the Exclusive Area Requirement Scope.
 * @param worker__ \b In. Thread worker.
 */
  #define REDAWorkerExclusiveAreaRequirementsScope_finishExclusionByLevel(worker__) \
    while (exclusionAreaByLevelCounter__ > 0) { \
        REDAWorkerExclusiveAreaRequirementsScope_leaveExclusionByLevel( \
                worker__); \
    } \
    REDAWorkerExclusiveAreaRequirementsScope_end()

#else

  #define REDAWorkerExclusiveAreaRequirementsScope_begin( \
          maxForbiddenEANames__, \
          maxForbiddenEALevels__)
  #define REDAWorkerExclusiveAreaRequirementsScope_enterExclusionByName( \
          exclusiveAreaName__, \
          checkRequirementIsCurrentlyMet__, \
          worker__)
  #define REDAWorkerExclusiveAreaRequirementsScope_leaveExclusionByName( \
          worker__)
  #define REDAWorkerExclusiveAreaRequirementsScope_enterExclusionByLevel( \
          exclusiveAreaLevel__, \
          checkRequirementIsCurrentlyMet__, \
          worker__)
  #define REDAWorkerExclusiveAreaRequirementsScope_leaveExclusionByLevel( \
          worker__)
  #define REDAWorkerExclusiveAreaRequirementsScope_end()
  #define REDAWorkerExclusiveAreaRequirementsScope_startExclusionByName( \
          exclusiveAreaLevel__, \
          skipCondition__, \
          worker__)
  #define REDAWorkerExclusiveAreaRequirementsScope_finishExclusionByName( \
          worker__)
  #define REDAWorkerExclusiveAreaRequirementsScope_startExclusionByLevel( \
          exclusiveAreaLevel__, \
          skipCondition__, \
          worker__)
  #define REDAWorkerExclusiveAreaRequirementsScope_finishExclusionByLevel( \
          worker__)

#endif /* RTI_DEBUG_ENABLE_EA_EXCLUSION_REQUIREMENTS */



#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* reda_worker_impl_h */
