/**
 * (c) Copyright, Real-Time Innovations, 2022
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 *
 * Description
 * -----------
 * RTI definitions of cryptography library structures and macros to enable
 * support for wolfSSL 5.5.0 and 5.5.1.
 * A user application does not need to include this header file. It's only
 * intended for internal use by other connextdds modules.
 * =========================================================================
 */

#ifndef security_crypto_lib_adapter_wolfssl55_h
#define security_crypto_lib_adapter_wolfssl55_h
#if defined(RTI_WOLFSSL) \
    && (LIBWOLFSSL_VERSION_HEX >= RTI_WOLFSSL_VERSION_5_5_0)

/*
 * Needed because the values returned by X509_get_signature_nid differ between
 * OpenSSL and wolfSSL 4.7.0. wolfSSL 5.0.0 added the NID values to its
 * compatibility layer.
 */
#define RTI_SECURITY_SIGNATURE_ID_ECDSA_SHA384 NID_ecdsa_with_SHA384
#define RTI_SECURITY_SIGNATURE_ID_ECDSA_SHA256 NID_ecdsa_with_SHA256
#define RTI_SECURITY_SIGNATURE_ID_RSA_PKCS_SHA256 NID_sha256WithRSAEncryption

#endif /* RTI_WOLFSSL */
#endif /* security_crypto_lib_adapter_wolfssl55_h */